/*
	This file is part of the smb library
    Copyright (C) 1999  Nicolas Brodu
    brodu@iie.cnam.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include "NetBIOSPacket.h"
#ifdef DEBUG
#include <string.h>
#include <iostream.h>
#endif

NetBIOSPacket::NetBIOSPacket()
{
	modified=0;
}

// Nothing to do on this level, subclasses might use it
void NetBIOSPacket::update()
{
	modified=0;
}

// see RFC 1001/2 for this weird conversion
char* NetBIOSPacket::NBName(const char *name)
{
	uint8 len=strlen(name); // First label must be 32 bytes long
	if (len>16) len=16;   // no pointers can't be used in SMB
	int8 allocated=0;	// if we have to allocate a new string
	if (len<16)
	{
		char* name2=new char[16];
		strcpy(name2,name);
		for (int i=len; i<16; i++) name2[i]=' '; // Fill with spaces
		allocated=1;
		len=16;
		name=name2;
	}
	char *ret = new char[256]; // Max size (label size+label+scope)
	ret[0]=32;	// number of bytes in this label
	// I'd like to replace all the array access with (*array++)
	// but I don't know how CPUs with parity would react...
#if DEBUG >= 5
	cout<<"name : "<<name<<"\nlen : "<<(int)len<<"\n";
#endif
	for (int i=0; i<16; i++)
	{
		// Hope  and so won't matter...
		uint8 n=name[i];
		if ((n>='a') && (n<='z')) n-=32;
		ret[2*i+1]=((n>>4)&0xF)+0x41;
		ret[2*i+2]=(n&0xF)+0x41;
	}
#if DEBUG >= 5
	cout<<"name : "<<name<<"\nlen : "<<(int)len<<"\n";
#endif
	
	char *dom=DOMAIN; //NetBIOS domain
	uint32 posLen=33;
	uint32 posCur=34;
	uint16 len2=strlen(dom);
	uint16 lenCur=0;
    for (int i=0; i<len2; i++)
    {
    	if ((dom[i] != '.') && (lenCur<63))
    	{
    		lenCur++;
    		ret[posCur++]=dom[i];
    	}
    	else
    	{
    		ret[posLen]=lenCur;
    		lenCur=0;
    		posLen=posCur++;
    	}
    }
	ret[posLen]=lenCur;
	ret[posCur]=0;	//end
	if (allocated) delete name;
	errno=0;
	return ret;
}
